# Copyright (c) 2008-2010 by Yaco Sistemas <precio@yaco.es>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.

from os import path
from StringIO import StringIO
import zipfile

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.files.storage import FileSystemStorage
from django.template import Template, Context
from django.test import TestCase, Client

from oot.models import WriterTemplate


class OOTTests(TestCase):

    def setUp(self):
        # replace template's storage with my own test storage
        my_media_location = path.join(settings.BASEDIR, 'oot', 'tests_media')
        my_storage = FileSystemStorage(location=my_media_location)
        template_field = WriterTemplate._meta.get_field_by_name('template')[0]
        template_field.storage = my_storage

        # create a WriterTemplate object to be rendered by itself
        self.wt_ct = ContentType.objects.get(app_label='oot',
                                             name='writer template')
        wt = WriterTemplate()
        wt.title = 'Writer Template'
        wt.template = 'writertemplate.odt'
        wt.content_type = self.wt_ct
        wt.save()
        self.wt = wt

    def assertODTContains(self, data):
        data_file = StringIO()
        data_file.write(data)
        odt_file = zipfile.ZipFile(data_file, 'r')
        content = odt_file.read('content.xml')
        odt_file.close()
        data_file.close()
        return self.assert_("Writer Template" in content)

    def test_render_writertemplate_view(self):
        c = Client()
        response = c.get('/oot/render_writer/%s/%s/' % (self.wt.id, self.wt.id))
        self.assertODTContains(response.content)

    def test_templatetag(self):
        t = Template('''{% load oot_templates %}{% writer_templates obj %}''')
        c = Context({'obj': self.wt})
        content = t.render(c)
        self.assert_("Writer Template" in content)



# Copyright (c) 2008-2010 by Yaco Sistemas <precio@yaco.es>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls.defaults import *

urlpatterns = patterns('oot.views',
    (r'^render_writer/(?P<template_id>\d+)/(?P<object_id>\d+)/$',
     'render_writer_template'),
)



# Copyright (c) 2008-2010 by Yaco Sistemas <precio@yaco.es>
#
# This file is part of Merengue.
#
# Merengue is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Merengue is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Merengue.  If not, see <http://www.gnu.org/licenses/>.

from django.template import Library
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site

from oot.models import WriterTemplate
register = Library()


@register.inclusion_tag('writer_templates.html')
def writer_templates(obj):
    result = {'obj_id': obj.id}
    content_type = ContentType.objects.get_for_model(obj)
    templates = WriterTemplate.objects.filter(content_type=content_type)
    result['templates'] = templates
    return result


@register.inclusion_tag('render_image.xml')
def render_image(image_path):
    if image_path[:4] != 'http':
        domain = Site.objects.all()[0].domain
        image_path = 'http://%s%s' % (domain, image_path)
    return {'image_path': image_path}



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 by Yaco Sistemas <precio@yaco.es>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.

from cStringIO import StringIO
import zipfile

from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.template import Context, Template
from django.utils.encoding import smart_str

from oot.models import WriterTemplate


def render(template, object_id):
    """Render the object with the given id in an OpenOffice Writer
    document using the Django template system and return it."""
    obj = template.content_type.get_object_for_this_type(id=int(object_id))

    # Don't recalculate the template if is cached
    output_buffer = StringIO()
    output_file = zipfile.ZipFile(output_buffer, 'w')
    odt_file = zipfile.ZipFile(template.template.path, 'r')
    for file_name in odt_file.namelist():
        data = odt_file.read(file_name)
        if file_name == 'content.xml':
            # hook for Django templates in the contents file
            t = cache.get('oot_template_%s' % template.id)
            if t is None:
                data = data.replace('&quot;', '"')
                t = Template(data)
            else:
                cache.set('oot_template_%s' % template.id, t, 18000)
            c = Context({template.content_type.model: obj})
            data = t.render(c)
        output_file.writestr(file_name, smart_str(data))
    output_file.close()
    odt_file.close()

    return output_buffer.getvalue()


def render_writer_template(request, template_id, object_id):
    template = get_object_or_404(WriterTemplate, id=int(template_id))
    try:
        data = render(template, object_id)
    except ObjectDoesNotExist:
        raise Http404

    file_name = '%s.odt' % smart_str(template.title)
    response = HttpResponse(mimetype="application/vnd.oasis.opendocument.text")
    response['Content-Disposition'] = 'attachment;filename="%s"' % file_name
    response.write(data)
    return response



# Copyright (c) 2008-2010 by Yaco Sistemas <precio@yaco.es>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this programe.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _


class WriterTemplate(models.Model):
    title = models.CharField(_(u'title'), max_length=100)
    template = models.FileField(_(u'template'), upload_to='oot')
    content_type = models.ForeignKey(ContentType,
        verbose_name=_(u'content type'))

    class Meta:
        verbose_name = _(u'writer template')
        verbose_name_plural = _(u'writer templates')

    def __unicode__(self):
        return self.title
